/*
 * Copyright (c) 2016, sobomax
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * 
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _ELPERIODIC_H_
#define _ELPERIODIC_H_

enum prdic_det_type {PRDIC_DET_FREQ, PRDIC_DET_PHASE};

void *prdic_init(double freq_hz, double off_from_now);
int prdic_procrastinate(void *prdic_inst);
time_t prdic_getncycles_ref(void *);
void prdic_set_fparams(void *, double);
void prdic_set_epoch(void *, struct timespec *);
void prdic_free(void *prdic_inst);
int prdic_addband(void *prdic_inst, double freq_hz);
void prdic_useband(void *prdic_inst, int bnum);
enum prdic_det_type prdic_set_det_type(void *prdic_inst, int bnum,
  enum prdic_det_type);
double prdic_getload(void *);
int prdic_islocked(void *);
int prdic_CFT_enable(void *prdic_inst, int signum);
int prdic_call_from_thread(void *, void (*)(void *), void *);

#endif /* _ELPERIODIC_H_ */
